"use client";

import Loading from "@/app/loading";
import { SearchIcon } from "@chakra-ui/icons";
import {
  Box,
  Button,
  Fade,
  Flex,
  Icon,
  Input,
  InputGroup,
  InputLeftElement,
  Text,
  useColorModeValue,
} from "@chakra-ui/react";
import React, { useMemo, useState } from "react";
import { IoRefresh } from "react-icons/io5";
import Card from "./card";
import Pagination from "../../components/pagination";

export interface CardListProps {
  id: number;
  full_name: string;
  contact_information: string;
  gis_tagging: string;
  contract_no_or_consumer_no: string;
  address: string;
  meter_no: string;
  property_type: string;
  meters_installed: string;
  premises_type: string;
  premises_in_use: number | string;
  meter_exist_on_site: number | string;
  additional_meter: string;
  additional_meter_req: number | string;
  additional_meter_applied: number | string;
  offline: boolean;
  created_at: string;
  files: number[];
}

const CardList = ({
  data = [],
  isLoading = false,
  handleSync,
}: {
  data?: CardListProps[];
  isLoading?: boolean;
  handleSync: () => void;
}) => {
  const [search, setSearch] = useState("");
  const [openCardIndex, setOpenCardIndex] = useState<number | null>(null);
  const [currentPage, setCurrentPage] = useState(1);
  const itemsPerPage = 10;

  const handleSearchChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setSearch(event.target.value);
  };

  const toggleCard = (index: number) => {
    setOpenCardIndex(openCardIndex === index ? null : index);
  };

  const filteredData = useMemo(() => {
    if (!Array.isArray(data)) return [];
    if (!search) return data;
    const lowercasedSearch = search.toLowerCase();
    return data.filter(
      (consumer) =>
        consumer?.full_name?.toLowerCase()?.includes(lowercasedSearch) ||
        consumer?.contact_information?.toLowerCase()?.includes(lowercasedSearch)
    );
  }, [search, data]);

  const startIndex = (currentPage - 1) * itemsPerPage;
  const endIndex = startIndex + itemsPerPage;
  const currentData = Array.isArray(filteredData)
    ? filteredData.slice(startIndex, endIndex)
    : [];

  // Handle page changes
  const handlePageChange = (newPage: number) => {
    setCurrentPage(newPage);
  };

  const totalPages = Math.ceil(filteredData.length / itemsPerPage);

  return (
    <>
      <Flex alignItems="center" justifyContent="space-between">
        <Button
          onClick={handleSync}
          isDisabled={currentData?.some(
            (item: { offline: Boolean }) => item?.offline == true
          )}
        >
          <Icon as={IoRefresh} w="20px" h="20px" />
          &nbsp; Sync
        </Button>
        <InputGroup width="300px">
          <InputLeftElement pointerEvents="none">
            <SearchIcon color={useColorModeValue("black", "white")} />
          </InputLeftElement>
          <Input
            placeholder="Search..."
            variant="outline"
            focusBorderColor="blue.500"
            size="md"
            type="search"
            color={useColorModeValue("black", "white")}
            onChange={handleSearchChange}
          />
        </InputGroup>
      </Flex>
      <br />

      {isLoading ? (
        <Loading />
      ) : (
        <>
          {currentData?.length > 0 ? (
            currentData?.map((consumer, index) => (
              <Card
                key={index}
                consumer={consumer}
                isOpen={openCardIndex === index}
                onToggle={() => toggleCard(index)}
              />
            ))
          ) : (
            <Box mt="70px" display="flex" justifyContent="center">
              <Text>No Data Found</Text>
            </Box>
          )}
        </>
      )}

      {!isLoading && totalPages > 1 && (
        <Pagination
          currentPage={currentPage}
          totalPages={totalPages}
          onPageChange={handlePageChange}
        />
      )}
    </>
  );
};

export default React.memo(CardList);
